#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

/* custom font */
#include <Fonts/FreeMonoBold9pt7b.h>
#include <Adafruit_MLX90614.h>

#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 32
#define OLED_RESET -1

/* initialize the OLED display */
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

/* Laser Diode Pin */
const int Laser_Pin = 5;

/* initialize the thermometer */
Adafruit_MLX90614 mlx = Adafruit_MLX90614();

/* the temperature that will be shown on display */
int temp;

/* will indicate if the temperature will be shown is Celsius or Fahrenheit */
bool isCelsius = true;

/* a timer will help converting the temperature unit to Celsius from Fahrenheit and back every 5 seconds */
unsigned long previousMillis = 0;
const unsigned long interval = 5000;

void setup()
{
  Serial.begin(9600);
  
  /* a small delay is needed for the display to initialize */                
  delay(100);
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.clearDisplay();

  /* color WHITE for written pixels */
  display.setTextColor(WHITE);

  mlx.begin();
  
  /* the laser pin will be OUTPUT */
  pinMode(Laser_Pin, OUTPUT);
}

void loop()
{

  /* the laser will be always ON */
  digitalWrite(Laser_Pin, HIGH);

  /* current timestamp */
  unsigned long currentMillis = millis();

  /* manage the unit conversion after every 5 seconds */
  if (currentMillis - previousMillis >= interval)
  {
    previousMillis = currentMillis;
    isCelsius = !isCelsius;
  }

  if (isCelsius) {
    temp = mlx.readObjectTempC();
  } else {
    temp = mlx.readObjectTempF();
  }

  /* the temperature will be shown on PC through the UART interface */
  Serial.print("Temperature = ");
  Serial.println(temp);

  display.clearDisplay();

  /* print text 'TEMPERATURE' */
  display.setFont();
  display.setCursor(45, 5);
  display.println("TEMPERATURE");

  /* print the temperature */
  char string[10];
  /* temp needs to be converted to string */
  dtostrf(temp, 4, 0, string);
  
  /* custom font */
  display.setFont(&FreeMonoBold9pt7b);
  display.setCursor(22, 25);
  display.println(string);
  display.setCursor(90, 25);

  /* display the temperature unit */
  if (isCelsius)
    display.println("C");
  else
    display.println("F");

  /* revert to the default font */
  display.setFont(); 
  display.setCursor(78,15);
  display.cp437(true);
  /* display the temperature symbol */
  display.write(167);
  
  /* display the thermometer */
  /* draw a filled circle with center in (18, 27) and radius of 5 */
  display.fillCircle(18, 27, 5, WHITE);
    
  /* draw rounded rectangle with top left corner (16, 3), width 5, height 24, radius 2 */
  display.drawRoundRect(16, 3, 5, 24, 2, WHITE);

  for (int i = 3; i <= 18; i = i + 2) {
    display.drawLine(21, i, 22, i, WHITE);
  }
  
  /* show the temperature accurately on the thermometer */
  /* convert back from F to C for accuracy */
  if (!isCelsius) {
    temp = (temp - 32) * 5 / 9;
  }
  temp = temp * 0.43;
  display.drawLine(18, 23, 18, 23 - temp, WHITE);

  
  /* finally print everything on the display */
  display.display();
}
